package fr.numeco.analyser.service;

import fr.numeco.analyser.AnalyzerConfig;
import fr.numeco.misis.analyzer.service.AnalyserService;
import fr.numeco.misis.enums.AnalyseStatut;
import fr.numeco.misis.sitemap.service.SitemapService;
import fr.numeco.misis.suividesite.domain.Page;
import io.quarkus.logging.Log;
import io.quarkus.scheduler.Scheduled;
import static io.quarkus.scheduler.Scheduled.ConcurrentExecution.SKIP;
import io.quarkus.scheduler.ScheduledExecution;
import jakarta.enterprise.context.ApplicationScoped;
import lombok.RequiredArgsConstructor;

@ApplicationScoped
@RequiredArgsConstructor
public class AnalysisScheduler {

    private final SitemapService sitemapService;
    private final AnalyserService analyserService;
    private final AnalyzerConfig analyzerConfig;

    @Scheduled(identity = "AnalysisScheduler", cron = "{analyser.refresh.schedule.cron.expression}", concurrentExecution = SKIP, skipExecutionIf = AnalyserService.class)
    void scheduleGroupesDePagesAnalysis(final ScheduledExecution execution) {

        sitemapService.process()
                .subscribe().with(
                        this::log,
                        failure -> Log.error("Sitemap error", failure),
                        () -> {
                            analyserService.process(0, analyzerConfig.paginationSize()-1);
                            if(Log.isDebugEnabled()) {
                                Log.debug("Flux completed");
                            }
                        }
                )
                ;
    }

    private void log(Page page) {
        if(Log.isDebugEnabled()) {
            if (page.getAnalyseStatut() == AnalyseStatut.NOUVEAU) {
                Log.debugf("Page %s créée par le sitemap.", page.getUrl());
            } else {
                Log.debugf("Page %s mise à jour par le sitemap.", page.getId());
            }
        }
    }

}
