package fr.numeco.misis.suividesite.mapper;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.stream.Collectors;

import org.mapstruct.Mapper;
import org.mapstruct.MappingConstants;

import static fr.numeco.analyser.Constants.STATUSES_IN_ERROR;
import fr.numeco.misis.enums.Alert;
import fr.numeco.misis.enums.RessourceType;
import fr.numeco.misis.suividesite.domain.GroupeDePages;
import fr.numeco.misis.suividesite.domain.Page;
import fr.numeco.misis.suividesite.domain.Ressource;
import fr.numeco.misis.suividesite.domain.SuiviDeSite;
import fr.numeco.misis.suividesite.dto.GroupDetailsDto;
import fr.numeco.misis.suividesite.dto.WebsiteSummaryDto;

@Mapper(componentModel = MappingConstants.ComponentModel.JAKARTA)
public interface WebsiteSummaryMapper {

    default WebsiteSummaryDto toDto(final SuiviDeSite suiviDeSite, final LocalDateTime nextAnalysis) {
        final Long id = suiviDeSite.getId();
        final String name = suiviDeSite.getName();
        final String url = suiviDeSite.getUrl();
        List<Long> suiviInError = new ArrayList<>();
        LocalDateTime lastAnalysis = null;

        final Map<Long, GroupDetailsDto> groups = new TreeMap<>();
        long pageCount = 0L;
        for (GroupeDePages groupeDePages : suiviDeSite.getGroupesDePages()) {
            List<Long> groupInError = new ArrayList<>();
            LocalDateTime groupLastAnalysis = null;
            Set<RessourceType> availableRessourceTypes = new TreeSet<>();

            for (Page page : groupeDePages.getPages()) {
                if (STATUSES_IN_ERROR.contains(page.getAnalyseStatut())) {
                    suiviInError.add(page.getId());
                    groupInError.add(page.getId());
                }
                if (Objects.nonNull(page.getAnalysedAt()) && (groupLastAnalysis == null || page.getAnalysedAt().isAfter(groupLastAnalysis))) {
                    groupLastAnalysis = page.getAnalysedAt();
                }
                if (page.getRessources() != null) {
                    availableRessourceTypes.addAll(page.getRessources().stream()
                            .map(Ressource::getType)
                            .filter(Objects::nonNull)
                            .collect(Collectors.toSet()));
                }
            }

            if (groupLastAnalysis != null && (lastAnalysis == null || groupLastAnalysis.isAfter(lastAnalysis))) {
                lastAnalysis = groupLastAnalysis;
            }

            Alert alert = null;
            if (!groupInError.isEmpty() && groupInError.size() < groupeDePages.getPages().size()) {
                alert = Alert.WARNING;
            } else if (!groupInError.isEmpty() && groupInError.size() == groupeDePages.getPages().size()) {
                alert = Alert.ERROR;
            }

            pageCount += groupeDePages.getPages().size();
            groups.put(groupeDePages.getId(), new GroupDetailsDto(groupeDePages.getName(), availableRessourceTypes, groupeDePages.getCreatedDate().atStartOfDay(), groupLastAnalysis, groupInError.size(), groupeDePages.getPages().size(), alert));
        }

        Alert alert = null;
        if (!suiviInError.isEmpty() && suiviInError.size() < pageCount) {
            alert = Alert.WARNING;
        } else if (!suiviInError.isEmpty() && suiviInError.size() == pageCount) {
            alert = Alert.ERROR;
        }

        return new WebsiteSummaryDto(id, name, url, lastAnalysis, nextAnalysis, groups, pageCount, alert, (long) suiviInError.size());
    }
}
