package fr.numeco.misis.suividesite.repository;

import java.util.List;
import java.util.Optional;

import fr.numeco.misis.suividesite.domain.GroupeDePages;
import fr.numeco.misis.suividesite.domain.Page;
import io.quarkus.hibernate.orm.panache.PanacheQuery;
import io.quarkus.hibernate.orm.panache.PanacheRepository;
import jakarta.enterprise.context.ApplicationScoped;

@ApplicationScoped
public class PageRepository  implements PanacheRepository<Page> {
    private static final String QUERY_ANALYZABLE_PAGES = """
                SELECT p FROM Page p
                JOIN p.groupeDePages g
                WHERE p.analyseStatut = 'NOUVEAU' OR
                (
                    p.analyseStatut = 'TERMINEE' AND
                    (
                        (p.analysedAt < CURRENT_DATE AND g.periodiciteDuSuivi = 'QUOTIDIENNE')
                        OR (g.periodiciteDuSuivi = 'HEBDOMADAIRE' AND DATEADD(week, -1, CURRENT_DATE) > p.analysedAt)
                        OR (g.periodiciteDuSuivi = 'MENSUELLE' AND DATEADD(month, -1, CURRENT_DATE) > p.analysedAt)
                    )
                )
                ORDER BY p.modifiedAt ASC
            """;

    private static final String DELETE_OUTDATED_URLS = """
                DELETE FROM Page WHERE url NOT IN (?1) AND groupeDePages = ?2
            """;

    public PanacheQuery<Page> findAvailableByPeriodicity() {
        return find(QUERY_ANALYZABLE_PAGES);
    }

    public void filterOutdatedUrls(List<String> sitemapUrls, GroupeDePages groupeDePages) {
        delete(DELETE_OUTDATED_URLS, sitemapUrls, groupeDePages);
    }

    public Optional<Page> findByUrlAndGroupeDePages(String url, GroupeDePages groupeDePages) {
        return find("url = ?1 AND groupeDePages = ?2", url, groupeDePages).firstResultOptional();
    }
}
