package fr.numeco.misis.suividesite.service;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.function.Function;

import static fr.numeco.analyser.Constants.STATUSES_IN_ERROR;
import fr.numeco.misis.enums.RessourceType;
import static fr.numeco.misis.statistic.service.impl.DefaultStatisticService.IMAGE_THRESHOLD_IN_BYTES;
import fr.numeco.misis.suividesite.domain.GroupeDePages;
import fr.numeco.misis.suividesite.domain.Page;
import fr.numeco.misis.suividesite.domain.Ressource;
import fr.numeco.misis.suividesite.domain.SuiviDeSite;
import fr.numeco.misis.suividesite.dto.GroupDetailsDto;
import fr.numeco.misis.suividesite.dto.WebsiteFormDto;
import fr.numeco.misis.suividesite.dto.WebsiteSummaryDto;
import fr.numeco.misis.suividesite.dto.abst.WebsiteDetails;
import io.smallrye.mutiny.tuples.Tuple2;

public interface WebsiteService {

    List<WebsiteSummaryDto> getAllWebsitesVisibleToCurrentUser();

    void create(WebsiteFormDto websiteFormDto);

    WebsiteDetails getPagesDetails(Long suiviId, Long groupId);

    WebsiteDetails getResourcesDetails(Long suiviId, Long groupId, RessourceType type);

    WebsiteSummaryDto getWebsiteByIdVisibleToCurrentUser(Long suiviId);

    static Tuple2<Long, Map<Long, GroupDetailsDto>> getGroupDetails(SuiviDeSite suiviDeSite) {
        return getGroupDetails(suiviDeSite, null);
    }

    static Tuple2<Long, Map<Long, GroupDetailsDto>> getGroupDetails(SuiviDeSite suiviDeSite, Function<GroupeDePages, Void> function) {
        long pageCount = 0L;
        final Map<Long, GroupDetailsDto> groups = new TreeMap<>();
        LocalDateTime groupLastAnalysis = null;
        for (GroupeDePages groupeDePages : suiviDeSite.getGroupesDePages()) {
            int groupInError = 0;
            final Set<RessourceType> availableRessourceTypes = new TreeSet<>();

            for (Page page : groupeDePages.getPages()) {
                if (STATUSES_IN_ERROR.contains(page.getAnalyseStatut())) {
                    groupInError++;
                }
                if (Objects.nonNull(page.getAnalysedAt()) && (groupLastAnalysis == null || page.getAnalysedAt().isAfter(groupLastAnalysis))) {
                    groupLastAnalysis = page.getAnalysedAt();
                }
                for(Ressource ressource : page.getRessources()) {
                    if(Objects.nonNull(ressource.getType()) && (!RessourceType.IMAGE.equals(ressource.getType()) || ressource.getSizeInBytes() > IMAGE_THRESHOLD_IN_BYTES)) {
                        availableRessourceTypes.add(ressource.getType());
                    }
                }

            }
            if (Objects.nonNull(function)) {
                function.apply(groupeDePages);
            }

            pageCount += groupeDePages.getPages().size();
            groups.put(groupeDePages.getId(), new GroupDetailsDto(groupeDePages.getName(), availableRessourceTypes, groupeDePages.getCreatedDate().atStartOfDay(), groupLastAnalysis, groupInError, groupeDePages.getPages().size(), null));
        }

        return Tuple2.of(pageCount, groups);
    }
}
