package fr.numeco.misis.suividesite.resource;

import java.util.regex.Matcher;

import static org.hamcrest.CoreMatchers.is;
import static org.jboss.resteasy.reactive.RestResponse.StatusCode.CREATED;
import org.junit.jupiter.api.Test;

import fr.numeco.misis.TestEmailRoleAugmentor;
import io.quarkus.test.common.http.TestHTTPEndpoint;
import io.quarkus.test.junit.QuarkusTest;
import io.quarkus.test.security.TestSecurity;
import static io.restassured.RestAssured.given;
import static io.restassured.RestAssured.when;

@QuarkusTest
@TestHTTPEndpoint(WebsiteResource.class)
class WebsiteResourceIntegrationTest {

    protected final String existingWebsite = "/{suiviId}";

    @Test
    @TestSecurity(user = "admin@test.fr", augmentors = TestEmailRoleAugmentor.class)
    void get_shouldReturnArrayOfWebsiteWhenUserIsAuthenticated() {
        when().get()
                .then()
                .statusCode(200)
                .body(is(Matcher.quoteReplacement(
                        "[{\"id\":1,\"name\":\"Site d'exemple\",\"url\":\"https://www.site-web-d-exemple-inexistant.fr/\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"1\":{\"name\":\"Groupe n°1\",\"availableRessourceTypes\":[\"IMAGE\",\"PDF\"],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"},\"51\":{\"name\":\"Groupe n°2\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"},\"101\":{\"name\":\"Groupe n°3\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"}},\"pageCount\":60,\"alert\":\"ERROR\",\"errorCount\":60},{\"id\":101,\"name\":\"Gitlab Page\",\"url\":\"https://misis-test-website-pub-numeco-misis-319bda392a27d3f07057d9125aa.gitlab-pages.din.developpement-durable.gouv.fr/\",\"lastAnalysis\":\"2999-01-01T00:00:00\",\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"201\":{\"name\":\"Mes Gitlab Pages\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2999-01-01T00:00:00\",\"errorCount\":0,\"pageCount\":4,\"alert\":null}},\"pageCount\":4,\"alert\":null,\"errorCount\":0},{\"id\":151,\"name\":\"Spote\",\"url\":\"https://spote.developpement-durable.gouv.fr/\",\"lastAnalysis\":null,\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"251\":{\"name\":\"Sitemap Spote\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":null,\"errorCount\":0,\"pageCount\":0,\"alert\":null}},\"pageCount\":0,\"alert\":null,\"errorCount\":0},{\"id\":201,\"name\":\"Test API\",\"url\":\"https://accenture.fr\",\"lastAnalysis\":null,\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"301\":{\"name\":\"Groupe API 1\",\"availableRessourceTypes\":[],\"createdDate\":\"2025-04-28T00:00:00\",\"lastAnalysis\":null,\"errorCount\":0,\"pageCount\":1,\"alert\":null}},\"pageCount\":1,\"alert\":null,\"errorCount\":0},{\"id\":51,\"name\":\"Site partagé par John Doe (un@test.fr)\",\"url\":\"https://www.john-doe-site-web.fr/\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"151\":{\"name\":\"Default\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":1,\"pageCount\":1,\"alert\":\"ERROR\"}},\"pageCount\":1,\"alert\":\"ERROR\",\"errorCount\":1}]")));
    }

    @Test
    @TestSecurity(user = "created@test.fr", augmentors = TestEmailRoleAugmentor.class)
    void
            post_shouldCreateWebsiteWhenCompleteFormIsProvided() {
        final String requestBodyWithForm = """
        {
          "name": "Test API",
          "url": "https://accenture.fr",
          "groupOfPages": [
            {
              "name": "Groupe API 1",
              "periodiciteDuSuivi": "QUOTIDIENNE",
              "methodeDeCreationDeGroupe": "MANUELLE",
              "pages": [
                {
                  "url": "https://accenture.com"
                }
              ]
            }
          ]
        }""";

        given()
                .header("Content-Type", "application/json")
                .body(requestBodyWithForm)
                .when().post()
                .then()
                .statusCode(CREATED)
                .body(is(""));
    }

    @Test
    @TestSecurity(user = "admin@test.fr", augmentors = TestEmailRoleAugmentor.class)
    void get_shouldReturnWebsiteWhenExistingIdIsProvided() {
        given()
                .pathParams("suiviId", 1)
                .get(existingWebsite)
                .then()
                .statusCode(200)
                .body(is(Matcher.quoteReplacement(
                        "{\"id\":1,\"name\":\"Site d'exemple\",\"url\":\"https://www.site-web-d-exemple-inexistant.fr/\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"nextAnalysis\":\"2000-01-01T01:00:00\",\"groups\":{\"1\":{\"name\":\"Groupe n°1\",\"availableRessourceTypes\":[\"IMAGE\",\"PDF\"],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"},\"51\":{\"name\":\"Groupe n°2\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"},\"101\":{\"name\":\"Groupe n°3\",\"availableRessourceTypes\":[],\"createdDate\":\"2000-01-01T00:00:00\",\"lastAnalysis\":\"2000-01-02T00:00:00\",\"errorCount\":20,\"pageCount\":20,\"alert\":\"ERROR\"}},\"pageCount\":60,\"alert\":\"ERROR\",\"errorCount\":60}")));
    }

    @Test
    @TestSecurity(user = "admin@test.fr", augmentors = TestEmailRoleAugmentor.class)
    void get_shouldReturnNotFoundWhenNotExistingIdIsProvided() {
        given()
                .pathParams("suiviId", -1)
                .get(existingWebsite)
                .then()
                .statusCode(404)
                .body("status", is("NOT_FOUND"))
                .body("message", is("Entity with id -1 not found"));
    }

}
